@@ -5,7 +5,7 @@ Support module for test_cursor[_async].py
 from __future__ import annotations
 
 import re
-from typing import Any, Match
+from typing import Any
 
 import pytest
 
@@ -15,12 +15,10 @@ from psycopg.rows import RowMaker
 
 @pytest.fixture(scope="session")
 def _execmany(svcconn):
-    svcconn.execute(
-        """
+    svcconn.execute("""
         drop table if exists execmany;
         create table execmany (id serial primary key, num integer, data text)
-        """
-    )
+        """)
 
 
 @pytest.fixture(scope="function")
@@ -40,7 +38,7 @@ def ph(cur: Any, query: str) -> str:
 
     n = 1
 
-    def s(m: Match[str]) -> str:
+    def s(m: re.Match[str]) -> str:
         nonlocal n
         rv = f"${n}"
         n += 1
