@@ -17,8 +17,8 @@ from psycopg.types.hstore import register_hstore
 from psycopg.types.numeric import Int4
 
 from .utils import eur
-from .acompat import alist
-from ._test_copy import sample_binary  # noqa
+from .acompat import AEvent, alist, gather, spawn
+from ._test_copy import sample_binary  # noqa: F401
 from ._test_copy import AsyncFileWriter, ensure_table_async, py_to_raw
 from ._test_copy import sample_binary_rows, sample_records, sample_tabledef
 from ._test_copy import sample_text, sample_values, special_chars
@@ -94,12 +94,10 @@ async def test_copy_out_param(aconn, ph, params):
 @pytest.mark.parametrize("typetype", ["names", "oids"])
 async def test_read_rows(aconn, format, typetype):
     cur = aconn.cursor()
-    async with cur.copy(
-        """copy (
+    async with cur.copy("""
+        copy (
             select 10::int4, 'hello'::text, '{0.0,1.0}'::float8[]
-        ) to stdout (format %s)"""
-        % format.name
-    ) as copy:
+    ) to stdout (format %s)""" % format.name) as copy:
         copy.set_types(["int4", "text", "float8[]"])
         row = await copy.read_row()
         assert (await copy.read_row()) is None
@@ -580,12 +578,10 @@ async def test_copy_in_allchars(aconn):
             await copy.write_row((i, None, chr(i)))
         await copy.write_row((ord(eur), None, eur))
 
-    await cur.execute(
-        """
+    await cur.execute("""
 select col1 = ascii(data), col2 is null, length(data), count(*)
 from copy_in group by 1, 2, 3
-"""
-    )
+""")
     data = await cur.fetchall()
     assert data == [(True, True, 1, 256)]
 
@@ -972,6 +968,105 @@ async def test_copy_table_across(aconn_cls, dsn, faker, mode):
             faker.assert_record(got, want)
 
 
+async def test_copy_concurrency(aconn):
+    """
+    Test that copy operations hold the connection lock for the entire operation.
+
+    This test verifies the fix for the concurrency issue where AsyncCursor.copy()
+    was not holding the connection lock throughout the copy context, allowing
+    concurrent operations to interfere.
+    """
+    await aconn.execute("create temp table copy_concurrency_test (id int, data text)")
+
+    # Events to coordinate execution between copy task and workers
+    copy_entered = AEvent()
+    wrote_first = AEvent()
+    wrote_second = AEvent()
+    can_proceed = AEvent()
+
+    # Track execution order to verify workers run after copy completes
+    execution_log = []
+
+    async def copy_task():
+        """Copy task that writes two rows with controlled pauses."""
+        cur = aconn.cursor()
+        async with cur.copy("copy copy_concurrency_test from stdin") as copy:
+            # Pause after entering copy context
+            execution_log.append("entered_copy")
+            copy_entered.set()
+            await can_proceed.wait()
+
+            # Write first row and pause
+            await copy.write_row((1, "first"))
+            execution_log.append("wrote_row_1")
+            wrote_first.set()
+            await can_proceed.wait()
+
+            # Write second row and pause
+            await copy.write_row((2, "second"))
+            execution_log.append("wrote_row_2")
+            wrote_second.set()
+            await can_proceed.wait()
+
+        # Copy context exited, lock should now be released
+        execution_log.append("exited_copy")
+
+    async def worker_task():
+        """
+        Worker that attempts to execute a query on a different cursor.
+        Should block until copy completes due to connection lock.
+        """
+        # Try to execute on another cursor - this should block until copy exits
+        worker_cur = aconn.cursor()
+        await worker_cur.execute("select 1")
+        execution_log.append("worker_completed")
+
+    # Start the copy task
+    t_copy = spawn(copy_task)
+
+    # Wait for copy to enter, then spawn first worker
+    await copy_entered.wait()
+    t_worker1 = spawn(worker_task)
+
+    # Allow copy to proceed to write first row
+    can_proceed.set()
+    can_proceed.clear()
+    await wrote_first.wait()
+
+    # Spawn second worker after first row
+    t_worker2 = spawn(worker_task)
+
+    # Allow copy to proceed to write second row
+    can_proceed.set()
+    can_proceed.clear()
+    await wrote_second.wait()
+
+    # Spawn third worker after second row
+    t_worker3 = spawn(worker_task)
+
+    # Allow copy to exit
+    can_proceed.set()
+
+    # Wait for all tasks to complete
+    await gather(t_copy, t_worker1, t_worker2, t_worker3)
+
+    # Verify the data was written correctly
+    cur = await aconn.execute("select * from copy_concurrency_test order by id")
+    rows = await cur.fetchall()
+    assert rows == [(1, "first"), (2, "second")]
+
+    # Verify that all workers completed AFTER copy exited
+    assert execution_log == [
+        "entered_copy",
+        "wrote_row_1",
+        "wrote_row_2",
+        "exited_copy",
+        "worker_completed",
+        "worker_completed",
+        "worker_completed",
+    ]
+
+
 class DataGenerator:
     def __init__(self, conn, nrecs, srec, offset=0, block_size=8192):
         self.conn = conn
