@@ -5,6 +5,7 @@ Tests common to psycopg.AsyncCursor and its subclasses.
 import weakref
 import datetime as dt
 from typing import Any
+from contextlib import aclosing
 
 import pytest
 from packaging.version import parse as ver
@@ -14,12 +15,15 @@ from psycopg import pq, rows, sql
 from psycopg.adapt import PyFormat
 from psycopg.types import TypeInfo
 
+from . import _test_cursor
 from .utils import raiseif
-from .acompat import aclosing, alist, anext, gather, spawn
+from .acompat import alist, gather, spawn
 from .fix_crdb import crdb_encoding
-from ._test_cursor import _execmany, execmany, my_row_factory, ph  # noqa: F401
+from .test_adapt import make_loader
+from ._test_cursor import my_row_factory, ph
 
-execmany = execmany  # avoid F811 underneath
+execmany = _test_cursor.execmany  # avoid F811 underneath
+_execmany = _test_cursor._execmany  # needed by the execmany fixture
 
 
 cursor_classes = [psycopg.AsyncCursor, psycopg.AsyncClientCursor]
@@ -158,6 +162,14 @@ async def test_execute_sql(aconn):
     assert (await cur.fetchone()) == ("hello",)
 
 
+async def test_next(aconn):
+    cur = aconn.cursor()
+    await cur.execute("select 1")
+    assert await anext(cur) == (1,)
+    with pytest.raises(StopAsyncIteration):
+        await anext(cur)
+
+
 async def test_query_parse_cache_size(aconn):
     cur = aconn.cursor()
     cls = type(cur)
@@ -211,6 +223,28 @@ async def test_execute_many_results(aconn):
     assert cur.nextset() is None
 
 
+async def test_set_results(aconn):
+    cur = aconn.cursor()
+
+    with pytest.raises(IndexError):
+        await cur.set_result(0)
+
+    await cur.execute("select 'foo'; select generate_series(1,3)")
+    assert await cur.set_result(0) is cur
+    assert (await cur.fetchall()) == [("foo",)]
+    assert cur.rowcount == 1
+
+    assert await cur.set_result(-1) is cur
+    assert (await cur.fetchall()) == [(1,), (2,), (3,)]
+    assert cur.rowcount == 3
+
+    with pytest.raises(IndexError):
+        await cur.set_result(2)
+
+    with pytest.raises(IndexError):
+        await cur.set_result(-3)
+
+
 async def test_execute_sequence(aconn):
     cur = aconn.cursor()
     rv = await cur.execute(
@@ -523,13 +557,11 @@ async def test_rownumber_none(aconn, query):
 
 async def test_rownumber_mixed(aconn):
     cur = aconn.cursor()
-    await cur.execute(
-        """
+    await cur.execute("""
 select x from generate_series(1, 3) x;
 set timezone to utc;
 select x from generate_series(4, 6) x;
-"""
-    )
+""")
     assert cur.rownumber == 0
     assert await cur.fetchone() == (1,)
     assert cur.rownumber == 1
@@ -923,3 +955,59 @@ async def test_row_maker_returns_none(aconn):
     assert await alist(cur) == recs
     stream = cur.stream(query)
     assert await alist(stream) == recs
+
+
+@pytest.mark.parametrize("count", [1, 3])
+async def test_results_after_execute(aconn, count):
+    async with aconn.cursor() as cur:
+        await cur.execute(
+            ";".join(f"select * from generate_series(1, {i})" for i in range(count))
+        )
+        ress = await alist(await res.fetchall() async for res in cur.results())
+        assert ress == [[(j + 1,) for j in range(i)] for i in range(count)]
+
+
+@pytest.mark.parametrize("count", [0, 1, 3])
+@pytest.mark.parametrize("returning", [False, True])
+async def test_results_after_executemany(aconn, count, returning):
+    async with aconn.cursor() as cur:
+        await cur.executemany(
+            ph(cur, "select * from generate_series(1, %s)"),
+            [(i,) for i in range(count)],
+            returning=returning,
+        )
+        ress = await alist(await res.fetchall() async for res in cur.results())
+        if returning:
+            assert ress == [[(j + 1,) for j in range(i)] for i in range(count)]
+        else:
+            assert ress == []
+
+
+async def test_change_loader_results(aconn):
+    cur = aconn.cursor()
+    # With no result
+    cur.adapters.register_loader("text", make_loader("1"))
+
+    await cur.execute("""
+        values ('foo'::text);
+        values ('bar'::text), ('baz');
+        values ('qux'::text);
+        """)
+    assert (await cur.fetchall()) == [("foo1",)]
+
+    cur.nextset()
+    assert (await cur.fetchone()) == ("bar1",)
+    cur.adapters.register_loader("text", make_loader("2"))
+    assert (await cur.fetchone()) == ("baz2",)
+    await cur.scroll(-2)
+    assert (await cur.fetchall()) == [("bar2",), ("baz2",)]
+
+    cur.nextset()
+    assert (await cur.fetchall()) == [("qux2",)]
+
+    # After the final result
+    assert not cur.nextset()
+    cur.adapters.register_loader("text", make_loader("3"))
+    assert (await cur.fetchone()) is None
+    await cur.set_result(0)
+    assert (await cur.fetchall()) == [("foo3",)]
